@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (c) 2014-2024 OpenRCT2 developers
+ * Copyright (c) 2014-2025 OpenRCT2 developers
  *
  * For a complete list of all authors, please refer to contributors.md
  * Interested in contributing? Visit https://github.com/OpenRCT2/OpenRCT2
@@ -10,12 +10,12 @@
 #include "ObjectRepository.h"
 
 #include "../Context.h"
+#include "../Diagnostic.h"
 #include "../OpenRCT2.h"
 #include "../PlatformEnvironment.h"
-#include "../common.h"
-#include "../config/Config.h"
 #include "../core/Console.hpp"
 #include "../core/DataSerialiser.h"
+#include "../core/EnumUtils.hpp"
 #include "../core/FileIndex.hpp"
 #include "../core/FileStream.h"
 #include "../core/Guard.hpp"
@@ -24,8 +24,8 @@
 #include "../core/MemoryStream.h"
 #include "../core/Numerics.hpp"
 #include "../core/Path.hpp"
+#include "../core/SawyerCoding.h"
 #include "../core/String.hpp"
-#include "../localisation/Localisation.h"
 #include "../localisation/LocalisationService.h"
 #include "../object/Object.h"
 #include "../park/Legacy.h"
@@ -33,15 +33,12 @@
 #include "../rct12/SawyerChunkReader.h"
 #include "../rct12/SawyerChunkWriter.h"
 #include "../scenario/ScenarioRepository.h"
-#include "../util/SawyerCoding.h"
-#include "../util/Util.h"
 #include "Object.h"
 #include "ObjectFactory.h"
 #include "ObjectList.h"
 #include "ObjectManager.h"
 #include "RideObject.h"
 
-#include <algorithm>
 #include <memory>
 #include <unordered_map>
 #include <vector>
@@ -79,7 +76,7 @@ class ObjectFileIndex final : public FileIndex<ObjectRepositoryItem>
 {
 private:
     static constexpr uint32_t MAGIC_NUMBER = 0x5844494F; // OIDX
-    static constexpr uint16_t VERSION = 29;
+    static constexpr uint16_t VERSION = 30;
     static constexpr auto PATTERN = "*.dat;*.pob;*.json;*.parkobj";
 
     IObjectRepository& _objectRepository;
@@ -87,11 +84,11 @@ private:
 public:
     explicit ObjectFileIndex(IObjectRepository& objectRepository, const IPlatformEnvironment& env)
         : FileIndex(
-            "object index", MAGIC_NUMBER, VERSION, env.GetFilePath(PATHID::CACHE_OBJECTS), std::string(PATTERN),
-            std::vector<std::string>{
-                env.GetDirectoryPath(DIRBASE::OPENRCT2, DIRID::OBJECT),
-                env.GetDirectoryPath(DIRBASE::USER, DIRID::OBJECT),
-            })
+              "object index", MAGIC_NUMBER, VERSION, env.GetFilePath(PATHID::CACHE_OBJECTS), std::string(PATTERN),
+              std::vector<std::string>{
+                  env.GetDirectoryPath(DIRBASE::OPENRCT2, DIRID::OBJECT),
+                  env.GetDirectoryPath(DIRBASE::USER, DIRID::OBJECT),
+              })
         , _objectRepository(objectRepository)
     {
     }
@@ -101,11 +98,11 @@ public:
     {
         std::unique_ptr<Object> object;
         auto extension = Path::GetExtension(path);
-        if (String::IEquals(extension, ".json"))
+        if (String::iequals(extension, ".json"))
         {
             object = ObjectFactory::CreateObjectFromJsonFile(_objectRepository, path, false);
         }
-        else if (String::IEquals(extension, ".parkobj"))
+        else if (String::iequals(extension, ".parkobj"))
         {
             object = ObjectFactory::CreateObjectFromZipFile(_objectRepository, path, false);
         }
@@ -163,6 +160,9 @@ protected:
             case ObjectType::FootpathSurface:
                 ds << item.FootpathSurfaceInfo.Flags;
                 break;
+            case ObjectType::PeepAnimations:
+                ds << item.PeepAnimationsInfo.PeepType;
+                break;
             default:
                 // Switch processes only ObjectType::Ride and ObjectType::SceneryGroup
                 break;
@@ -172,7 +172,7 @@ protected:
 private:
     bool IsTrackReadOnly(const std::string& path) const
     {
-        return String::StartsWith(path, SearchPaths[0]) || String::StartsWith(path, SearchPaths[1]);
+        return String::startsWith(path, SearchPaths[0]) || String::startsWith(path, SearchPaths[1]);
     }
 };
 
@@ -267,11 +267,11 @@ public:
         Guard::ArgumentNotNull(ori, GUARD_LINE);
 
         auto extension = Path::GetExtension(ori->Path);
-        if (String::IEquals(extension, ".json"))
+        if (String::iequals(extension, ".json"))
         {
             return ObjectFactory::CreateObjectFromJsonFile(*this, ori->Path, !gOpenRCT2NoGraphics);
         }
-        if (String::IEquals(extension, ".parkobj"))
+        if (String::iequals(extension, ".parkobj"))
         {
             return ObjectFactory::CreateObjectFromZipFile(*this, ori->Path, !gOpenRCT2NoGraphics);
         }
@@ -367,7 +367,7 @@ private:
     void SortItems()
     {
         std::sort(_items.begin(), _items.end(), [](const ObjectRepositoryItem& a, const ObjectRepositoryItem& b) -> bool {
-            return String::Compare(a.Name, b.Name) < 0;
+            return String::compare(a.Name, b.Name) < 0;
         });
 
         // Fix the IDs
@@ -470,6 +470,13 @@ private:
         }
     }
 
+    // 0x0098DA2C
+    static constexpr std::array<int32_t, 11> kLegacyObjectEntryGroupEncoding = {
+        CHUNK_ENCODING_RLE, CHUNK_ENCODING_RLE, CHUNK_ENCODING_RLE,    CHUNK_ENCODING_RLE,
+        CHUNK_ENCODING_RLE, CHUNK_ENCODING_RLE, CHUNK_ENCODING_RLE,    CHUNK_ENCODING_RLE,
+        CHUNK_ENCODING_RLE, CHUNK_ENCODING_RLE, CHUNK_ENCODING_ROTATE,
+    };
+
     static void SaveObject(
         std::string_view path, const RCTObjectEntry* entry, const void* data, size_t dataSize, bool fixChecksum = true)
     {
@@ -524,11 +531,11 @@ private:
 
         // Encode data
         ObjectType objectType = entry->GetType();
-        SawyerCodingChunkHeader chunkHeader;
-        chunkHeader.encoding = object_entry_group_encoding[EnumValue(objectType)];
+        SawyerCoding::ChunkHeader chunkHeader;
+        chunkHeader.encoding = kLegacyObjectEntryGroupEncoding[EnumValue(objectType)];
         chunkHeader.length = static_cast<uint32_t>(dataSize);
         uint8_t* encodedDataBuffer = Memory::Allocate<uint8_t>(0x600000);
-        size_t encodedDataSize = SawyerCodingWriteChunkBuffer(
+        size_t encodedDataSize = SawyerCoding::WriteChunkBuffer(
             encodedDataBuffer, reinterpret_cast<const uint8_t*>(data), chunkHeader);
 
         // Save to file
@@ -591,7 +598,7 @@ private:
         while (File::Exists(fullPath))
         {
             counter++;
-            fullPath = Path::Combine(userObjPath, String::StdFormat("%s-%02X%s", fileName.c_str(), counter, extension));
+            fullPath = Path::Combine(userObjPath, String::stdFormat("%s-%02X%s", fileName.c_str(), counter, extension));
         }
 
         return fullPath;
@@ -618,7 +625,7 @@ private:
             }
 
             // Convert to UTF-8 filename
-            return String::ConvertToUtf8(normalisedName, OpenRCT2::CodePage::CP_1252);
+            return String::convertToUtf8(normalisedName, OpenRCT2::CodePage::CP_1252);
         }
         else
         {
@@ -631,7 +638,7 @@ private:
         const ObjectRepositoryItem* item = FindObject(entry);
         if (item == nullptr)
         {
-            throw std::runtime_error(String::StdFormat("Unable to find object '%.8s'", entry->name));
+            throw std::runtime_error(String::stdFormat("Unable to find object '%.8s'", entry->name));
         }
 
         // Read object data from file
